pub struct Program<'info, T: Id + Clone> { /* private fields */ }Expand description
Type validating that the account is the given Program
The type has a programdata_address function that will return Option::Some
if the program is owned by the BPFUpgradeableLoader
which will contain the programdata_address property of the Program variant of the UpgradeableLoaderState enum.
Table of Contents
Basic Functionality
Checks:
account_info.key == expected_programaccount_info.executable == true
Example
#[program]
mod my_program {
fn set_admin_settings(...){...}
}
#[account]
#[derive(Default)]
pub struct AdminSettings {
...
}
#[derive(Accounts)]
pub struct SetAdminSettings<'info> {
#[account(mut, seeds = [b"admin"], bump)]
pub admin_settings: Account<'info, AdminSettings>,
#[account(constraint = program.programdata_address()? == Some(program_data.key()))]
pub program: Program<'info, MyProgram>,
#[account(constraint = program_data.upgrade_authority_address == Some(authority.key()))]
pub program_data: Account<'info, ProgramData>,
pub authority: Signer<'info>,
}The given program has a function with which the upgrade authority can set admin settings.
The required constraints are as follows:
programis the account of the program itself. Its constraint checks thatprogram_datais the account that contains the program’s upgrade authority. Implicitly, this checks thatprogramis a BPFUpgradeable program (program.programdata_address()?will beNoneif it’s not).program_data’s constraint checks that its upgrade authority is theauthorityaccount.- Finally,
authorityneeds to sign the transaction.
Out of the Box Types
Between the anchor_lang and anchor_spl crates,
the following Program types are provided out of the box:
Implementations
Methods from Deref<Target = AccountInfo<'info>>
pub fn signer_key(&self) -> Option<&Pubkey>
pub fn unsigned_key(&self) -> &Pubkey
pub fn lamports(&self) -> u64
pub fn try_lamports(&self) -> Result<u64, ProgramError>
pub fn data_len(&self) -> usize
pub fn try_data_len(&self) -> Result<usize, ProgramError>
pub fn data_is_empty(&self) -> bool
pub fn try_data_is_empty(&self) -> Result<bool, ProgramError>
pub fn try_borrow_lamports(&self) -> Result<Ref<'_, &mut u64>, ProgramError>
pub fn try_borrow_mut_lamports( &self, ) -> Result<RefMut<'_, &'a mut u64>, ProgramError>
pub fn try_borrow_data(&self) -> Result<Ref<'_, &mut [u8]>, ProgramError>
pub fn try_borrow_mut_data( &self, ) -> Result<RefMut<'_, &'a mut [u8]>, ProgramError>
Sourcepub fn realloc(
&self,
new_len: usize,
zero_init: bool,
) -> Result<(), ProgramError>
pub fn realloc( &self, new_len: usize, zero_init: bool, ) -> Result<(), ProgramError>
Realloc the account’s data and optionally zero-initialize the new memory.
Note: Account data can be increased within a single call by up to
solana_program::entrypoint::MAX_PERMITTED_DATA_INCREASE bytes.
Note: Memory used to grow is already zero-initialized upon program
entrypoint and re-zeroing it wastes compute units. If within the same
call a program reallocs from larger to smaller and back to larger again
the new space could contain stale data. Pass true for zero_init in
this case, otherwise compute units will be wasted re-zero-initializing.
pub fn assign(&self, new_owner: &Pubkey)
pub fn deserialize_data<T>(&self) -> Result<T, Box<ErrorKind>>where
T: DeserializeOwned,
pub fn serialize_data<T>(&self, state: &T) -> Result<(), Box<ErrorKind>>where
T: Serialize,